"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.extract = extract;

var _vm = _interopRequireDefault(require("vm"));

var _url = _interopRequireDefault(require("url"));

var _request = _interopRequireDefault(require("request"));

var _cheerio = _interopRequireDefault(require("cheerio"));

/**
 * A request promise wrapper.
 *
 * @param req Request function.
 * @param options Request options.
 * @return Request response and body.
 */
async function requestP(req, options) {
  const r = await new Promise((resolve, reject) => {
    req(options, (error, response, body) => {
      if (error) {
        reject(error);
        return;
      }

      resolve({
        response,
        body
      });
    });
  });
  return r;
}
/**
 * Extract script code from HTML code.
 *
 * @param html HTML code.
 * @return Script code.
 */


function extractScripts(html) {
  const r = [];

  const $ = _cheerio.default.load(html);

  $('script').each((elI, el) => {
    const data = $(el).html();

    if (data) {
      r.push(data);
    }
  });
  return r;
}
/**
 * Attempt to extract info from script.
 *
 * @param script Script code.
 * @return Result object or null.
 */


function extractScript(script) {
  let result = null;

  if (!/dlbutton/.test(script)) {
    return result;
  } // Create a context with wich to run code in
  // Creating the object with a null prototype is very important.
  // Prevents host variables from leaking into the sanbox.


  const ctx = _vm.default.createContext(Object.create(null));

  const runOpts = {
    timeout: 1000
  }; // Setup environment.

  const codePre = ['window = this;', 'document = (function(r) {', 'var elements = {', '"dlbutton": {},', '"fimage": {}', '};', 'r.getElementById = function(id) {', 'return elements[id];', '}', 'return r;', '})({});'].join('\n'); // Extract info from environment.

  const codePost = ['JSON.stringify({', '"dlbutton": document.getElementById("dlbutton").href', '})'].join('\n'); // Attempt to run code in sanbox and extract the info.

  try {
    // Run the pre script.
    _vm.default.runInContext(codePre, ctx, runOpts); // Run the script code.


    _vm.default.runInContext(script, ctx, runOpts); // Run the post script.
    // Force return value to be string, with concatenation, NOT casting.
    // This prevents any funny business from sandboxed code.
    // tslint:disable-next-line: restrict-plus-operands


    result = JSON.parse('' + _vm.default.runInContext(codePost, ctx, runOpts));
  } catch (err) {// Ignore failure.
  }

  return result;
}
/**
 * Extract file info from a URL.
 *
 * @param uri The URI to extract info from.
 * @param req Optional custom request function or null.
 * @return File info.
 */


async function extract(uri, req = null) {
  const requester = req || _request.default;
  const {
    response,
    body
  } = await requestP(requester, {
    url: uri
  });
  const {
    statusCode
  } = response;

  if (statusCode !== 200) {
    throw new Error(`Invalid status code: ${statusCode}`);
  }

  const bodyType = typeof body;

  if (bodyType !== 'string') {
    throw new Error(`Invalid body type: ${bodyType}`);
  }

  const scripts = extractScripts(body);
  let result = null;

  for (const script of scripts) {
    result = extractScript(script);

    if (result) {
      break;
    }
  }

  if (!result || !result.dlbutton) {
    throw new Error('Failed to extract info');
  }

  const download = _url.default.resolve(uri, result.dlbutton);

  const filename = decodeURI((_url.default.parse(download).pathname || '').split('/').pop() || '') || null;
  return {
    download,
    filename
  };
}
//# sourceMappingURL=extract.js.map
